@@ -38,12 +38,13 @@
 **
 ****************************************************************************/
 
+#include "android/devicediscoverybroadcastreceiver_p.h"
+#include "android/androidutils_p.h"
 #include "qbluetoothdevicediscoveryagent_p.h"
 #include <QCoreApplication>
 #include <QtCore/QLoggingCategory>
 #include <QtBluetooth/QBluetoothAddress>
 #include <QtBluetooth/QBluetoothDeviceInfo>
-#include "android/devicediscoverybroadcastreceiver_p.h"
 #include <QtCore/QJniEnvironment>
 #include <QtCore/private/qandroidextras_p.h>
 
@@ -59,7 +60,7 @@ enum {
 
 QBluetoothDeviceDiscoveryAgentPrivate::QBluetoothDeviceDiscoveryAgentPrivate(
     const QBluetoothAddress &deviceAdapter, QBluetoothDeviceDiscoveryAgent *parent) :
-    m_adapterAddress(deviceAdapter),
+    adapterAddress(deviceAdapter),
     m_active(NoScanActive),
     q_ptr(parent)
 {
@@ -120,9 +121,9 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
         return;
     }
 
-    if (!m_adapterAddress.isNull()
+    if (!adapterAddress.isNull()
         && adapter.callObjectMethod<jstring>("getAddress").toString()
-        != m_adapterAddress.toString()) {
+        != adapterAddress.toString()) {
         qCWarning(QT_BT_ANDROID) << "Incorrect local adapter passed.";
         lastError = QBluetoothDeviceDiscoveryAgent::InvalidBluetoothAdapterError;
         errorString = QBluetoothDeviceDiscoveryAgent::tr("Passed address is not a local device.");
@@ -189,6 +190,15 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
 
     qCDebug(QT_BT_ANDROID) << "Location turned on";
 
+    if (!(ensureAndroidPermission(BluetoothPermission::Scan) &&
+          ensureAndroidPermission(BluetoothPermission::Connect))) {
+        qCWarning(QT_BT_ANDROID) << "Device discovery start() failed due to missing permissions";
+        errorString = QBluetoothDeviceDiscoveryAgent::tr("Bluetooth adapter error");
+        lastError = QBluetoothDeviceDiscoveryAgent::UnknownError;
+        emit q->errorOccurred(lastError);
+        return;
+    }
+
     // install Java BroadcastReceiver
     if (!receiver) {
         // SDP based device discovery
@@ -225,18 +235,6 @@ void QBluetoothDeviceDiscoveryAgentPrivate::start(QBluetoothDeviceDiscoveryAgent
     if (requestedMethods & QBluetoothDeviceDiscoveryAgent::LowEnergyMethod) {
         // LE search only requested or classic discovery failed but lets try LE scan anyway
         Q_ASSERT(requestedMethods & QBluetoothDeviceDiscoveryAgent::LowEnergyMethod);
-
-        if (QNativeInterface::QAndroidApplication::sdkVersion() < 18) {
-            qCDebug(QT_BT_ANDROID) << "Skipping Bluetooth Low Energy device scan due to "
-                                      "insufficient Android version"
-                                     << QNativeInterface::QAndroidApplication::sdkVersion();
-            m_active = NoScanActive;
-            lastError = QBluetoothDeviceDiscoveryAgent::UnsupportedDiscoveryMethod;
-            errorString = QBluetoothDeviceDiscoveryAgent::tr("Low Energy Discovery not supported");
-            emit q->errorOccurred(lastError);
-            return;
-        }
-
         startLowEnergyScan();
     }
 }
@@ -301,14 +299,7 @@ void QBluetoothDeviceDiscoveryAgentPrivate::processSdpDiscoveryFinished()
             return;
         }
 
-        // start LE scan if supported
-        if (QNativeInterface::QAndroidApplication::sdkVersion() < 18) {
-            qCDebug(QT_BT_ANDROID) << "Skipping Bluetooth Low Energy device scan";
-            m_active = NoScanActive;
-            emit q->finished();
-        } else {
-            startLowEnergyScan();
-        }
+        startLowEnergyScan();
     }
 }
 
@@ -329,7 +320,7 @@ void QBluetoothDeviceDiscoveryAgentPrivate::processDiscoveredDevices(
     // the advertisement package.
     // If address is same but name different then we keep both entries.
 
-    for (int i = 0; i < discoveredDevices.size(); i++) {
+    for (qsizetype i = 0; i < discoveredDevices.size(); ++i) {
         if (discoveredDevices[i].address() == info.address()) {
             QBluetoothDeviceInfo::Fields updatedFields = QBluetoothDeviceInfo::Field::None;
             if (discoveredDevices[i].rssi() != info.rssi()) {
@@ -345,6 +336,13 @@ void QBluetoothDeviceDiscoveryAgentPrivate::processDiscoveredDevices(
                     discoveredDevices[i].setManufacturerData(key, info.manufacturerData(key));
                 updatedFields.setFlag(QBluetoothDeviceInfo::Field::ManufacturerData);
             }
+            if (discoveredDevices[i].serviceData() != info.serviceData()) {
+                qCDebug(QT_BT_ANDROID) << "Updating service data for" << info.address();
+                const QList<QBluetoothUuid> keys = info.serviceIds();
+                for (auto key : keys)
+                    discoveredDevices[i].setServiceData(key, info.serviceData(key));
+                updatedFields.setFlag(QBluetoothDeviceInfo::Field::ServiceData);
+            }
 
             if (lowEnergySearchTimeout > 0) {
                 if (discoveredDevices[i] != info) {
