@@ -37,6 +37,12 @@
 **
 ****************************************************************************/
 
+#include "qbluetoothservicediscoveryagent_p.h"
+#include "qbluetoothsocket_android_p.h"
+#include "android/servicediscoverybroadcastreceiver_p.h"
+#include "android/localdevicebroadcastreceiver_p.h"
+#include "android/androidutils_p.h"
+
 #include <QCoreApplication>
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/QLoggingCategory>
@@ -46,11 +52,6 @@
 #include <QtBluetooth/QBluetoothLocalDevice>
 #include <QtBluetooth/QBluetoothServiceDiscoveryAgent>
 
-#include "qbluetoothservicediscoveryagent_p.h"
-#include "qbluetoothsocket_android_p.h"
-#include "android/servicediscoverybroadcastreceiver_p.h"
-#include "android/localdevicebroadcastreceiver_p.h"
-
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(QT_BT_ANDROID)
@@ -85,13 +86,6 @@ QBluetoothServiceDiscoveryAgentPrivate::QBluetoothServiceDiscoveryAgentPrivate(
         }
     }
 
-    if (QNativeInterface::QAndroidApplication::sdkVersion() < 15)
-        qCWarning(QT_BT_ANDROID)
-            << "SDP not supported by Android API below version 15. Detected version: "
-            << QNativeInterface::QAndroidApplication::sdkVersion()
-            << "Service discovery will return empty list.";
-
-
     /*
       We assume that the current local adapter has been passed.
       The logic below must change once there is more than one adapter.
@@ -123,6 +117,15 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
 {
     Q_Q(QBluetoothServiceDiscoveryAgent);
 
+    if (!ensureAndroidPermission(BluetoothPermission::Connect)) {
+        qCWarning(QT_BT_ANDROID) << "Service discovery start() failed due to missing permissions";
+        error = QBluetoothServiceDiscoveryAgent::UnknownError;
+        errorString = QBluetoothServiceDiscoveryAgent::tr("Unable to perform SDP scan");
+        emit q->errorOccurred(error);
+        _q_serviceDiscoveryFinished();
+        return;
+    }
+
     if (!btAdapter.isValid()) {
         if (m_deviceAdapterAddress.isNull()) {
             error = QBluetoothServiceDiscoveryAgent::UnknownError;
@@ -142,29 +145,6 @@ void QBluetoothServiceDiscoveryAgentPrivate::start(const QBluetoothAddress &addr
         return;
     }
 
-    /* SDP discovery was officially added by Android API v15
-     * BluetoothDevice.getUuids() existed in earlier APIs already and in the future we may use
-     * reflection to support earlier Android versions than 15. Unfortunately
-     * BluetoothDevice.fetchUuidsWithSdp() and related APIs had some structure changes
-     * over time. Therefore we won't attempt this with reflection.
-     *
-     * TODO: Use reflection to support getUuuids() where possible.
-     * */
-    if (QNativeInterface::QAndroidApplication::sdkVersion() < 15) {
-        qCWarning(QT_BT_ANDROID) << "Aborting SDP enquiry due to too low Android API version (requires v15+)";
-
-        error = QBluetoothServiceDiscoveryAgent::UnknownError;
-        errorString = QBluetoothServiceDiscoveryAgent::tr("Android API below v15 does not support SDP discovery");
-
-        //abort any outstanding discoveries
-        sdpCache.clear();
-        discoveredDevices.clear();
-        emit q->errorOccurred(error);
-        _q_serviceDiscoveryFinished();
-
-        return;
-    }
-
     QJniObject inputString = QJniObject::fromString(address.toString());
     QJniObject remoteDevice =
             btAdapter.callObjectMethod("getRemoteDevice",
@@ -249,9 +229,11 @@ void QBluetoothServiceDiscoveryAgentPrivate::stop()
     discoveredDevices.clear();
 
     //kill receiver to limit load of signals
-    receiver->unregisterReceiver();
-    receiver->deleteLater();
-    receiver = nullptr;
+    if (receiver) {
+        receiver->unregisterReceiver();
+        receiver->deleteLater();
+        receiver = nullptr;
+    }
 
     Q_Q(QBluetoothServiceDiscoveryAgent);
     emit q->canceled();
@@ -262,12 +244,12 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
     const QBluetoothAddress &address, const QList<QBluetoothUuid> &uuids)
 {
     //don't leave more data through if we are not interested anymore
-    if (discoveredDevices.count() == 0)
+    if (discoveredDevices.isEmpty())
         return;
 
     //could not find any service for the current address/device -> go to next one
     if (address.isNull() || uuids.isEmpty()) {
-        if (discoveredDevices.count() == 1) {
+        if (discoveredDevices.size() == 1) {
             Q_Q(QBluetoothServiceDiscoveryAgent);
             QTimer::singleShot(4000, q, [this]() {
                 this->_q_fetchUuidsTimeout();
@@ -280,11 +262,11 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
 
     if (QT_BT_ANDROID().isDebugEnabled()) {
         qCDebug(QT_BT_ANDROID) << "Found UUID for" << address.toString()
-                               << "\ncount: " << uuids.count();
+                               << "\ncount: " << uuids.size();
 
         QString result;
-        for (int i = 0; i<uuids.count(); i++)
-            result += uuids.at(i).toString() + QStringLiteral("**");
+        for (const QBluetoothUuid &uuid : uuids)
+            result += uuid.toString() + QLatin1String("**");
         qCDebug(QT_BT_ANDROID) << result;
     }
 
@@ -303,7 +285,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
         //prefer second uuid set over first
         populateDiscoveredServices(pair.first, uuids);
 
-        if (discoveredDevices.count() == 1 && sdpCache.isEmpty()) {
+        if (discoveredDevices.size() == 1 && sdpCache.isEmpty()) {
             //last regular uuid data set from OS -> we finish here
             _q_serviceDiscoveryFinished();
         }
@@ -320,7 +302,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_processFetchedUuids(
 
         //the discovery on the last device cannot immediately finish
         //we have to grant the 2 seconds timeout delay
-        if (discoveredDevices.count() == 1) {
+        if (discoveredDevices.size() == 1) {
             Q_Q(QBluetoothServiceDiscoveryAgent);
             QTimer::singleShot(4000, q, [this]() {
                 this->_q_fetchUuidsTimeout();
@@ -353,9 +335,9 @@ void QBluetoothServiceDiscoveryAgentPrivate::populateDiscoveredServices(const QB
 
     //find SPP and custom uuid
     bool haveSppClass = false;
-    QList<int> customUuids;
+    QVarLengthArray<qsizetype> customUuids;
 
-    for (int i = 0; i < uuids.count(); i++) {
+    for (qsizetype i = 0; i < uuids.size(); ++i) {
         const QBluetoothUuid uuid = uuids.at(i);
 
         if (uuid.isNull())
@@ -385,7 +367,7 @@ void QBluetoothServiceDiscoveryAgentPrivate::populateDiscoveredServices(const QB
         return profileSequence;
     };
 
-    for (int i = 0; i < uuids.count(); i++) {
+    for (qsizetype i = 0; i < uuids.size(); ++i) {
         const QBluetoothUuid &uuid = uuids.at(i);
         if (uuid.isNull())
             continue;
@@ -464,8 +446,11 @@ void QBluetoothServiceDiscoveryAgentPrivate::populateDiscoveredServices(const QB
         //don't include the service if we already discovered it before
         if (!isDuplicatedService(serviceInfo)) {
             discoveredServices << serviceInfo;
-            //qCDebug(QT_BT_ANDROID) << serviceInfo;
-            emit q->serviceDiscovered(serviceInfo);
+            // Use queued connection to allow us finish the service discovery reporting;
+            // the application might call stop() when it has detected the service-of-interest,
+            // which in turn can cause the use of already released resources
+            QMetaObject::invokeMethod(q, "serviceDiscovered", Qt::QueuedConnection,
+                                      Q_ARG(QBluetoothServiceInfo, serviceInfo));
         }
     }
 }
@@ -485,9 +470,11 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_fetchUuidsTimeout()
     Q_ASSERT(sdpCache.isEmpty());
 
     //kill receiver to limit load of signals
-    receiver->unregisterReceiver();
-    receiver->deleteLater();
-    receiver = nullptr;
+    if (receiver) {
+        receiver->unregisterReceiver();
+        receiver->deleteLater();
+        receiver = nullptr;
+    }
     _q_serviceDiscoveryFinished();
 }
 
@@ -502,9 +489,11 @@ void QBluetoothServiceDiscoveryAgentPrivate::_q_hostModeStateChanged(QBluetoothL
         errorString = QBluetoothServiceDiscoveryAgent::tr("Device is powered off");
 
         //kill receiver to limit load of signals
-        receiver->unregisterReceiver();
-        receiver->deleteLater();
-        receiver = nullptr;
+        if (receiver) {
+            receiver->unregisterReceiver();
+            receiver->deleteLater();
+            receiver = nullptr;
+        }
 
         Q_Q(QBluetoothServiceDiscoveryAgent);
         emit q->errorOccurred(error);
